@@ -13,6 +13,24 @@ from keras import initializers, activations
 from keras.layers import *
 
 
+def integerize_shape(func):
+    """装饰器，保证input_shape一定是int或None
+    """
+    def convert(item):
+        if hasattr(item, '__iter__'):
+            return [convert(i) for i in item]
+        elif hasattr(item, 'value'):
+            return item.value
+        else:
+            return item
+
+    def new_func(self, input_shape):
+        input_shape = convert(input_shape)
+        return func(self, input_shape)
+
+    return new_func
+
+
 if keras.__version__[-2:] != 'tf' and keras.__version__ < '2.3':
 
     class Layer(keras.layers.Layer):
@@ -82,6 +100,7 @@ class MultiHeadAttention(Layer):
                  head_size,
                  key_size=None,
                  use_bias=True,
+                 scaled_dot_product=True,
                  kernel_initializer='glorot_uniform',
                  **kwargs):
         super(MultiHeadAttention, self).__init__(**kwargs)
@@ -90,6 +109,7 @@ class MultiHeadAttention(Layer):
         self.out_dim = heads * head_size
         self.key_size = key_size or head_size
         self.use_bias = use_bias
+        self.scaled_dot_product = scaled_dot_product
         self.kernel_initializer = initializers.get(kernel_initializer)
 
     def build(self, input_shape):
@@ -146,7 +166,7 @@ class MultiHeadAttention(Layer):
             pos_embeddings = K.permute_dimensions(inputs[n], (2, 0, 1))
             a = a + K.expand_dims(pos_embeddings, 0)
         # Attention（续）
-        if p_bias != 't5_relative':  # T5不用缩放
+        if self.scaled_dot_product:
             a = a / self.key_size**0.5
         a = sequence_masking(a, v_mask, 1, -1)
         if a_mask is not None:
@@ -174,6 +194,7 @@ class MultiHeadAttention(Layer):
             'head_size': self.head_size,
             'key_size': self.key_size,
             'use_bias': self.use_bias,
+            'scaled_dot_product': self.scaled_dot_product,
             'kernel_initializer': initializers.serialize(self.kernel_initializer),
         }
         base_config = super(MultiHeadAttention, self).get_config()
@@ -464,11 +485,10 @@ class FeedForward(Layer):
         self.use_bias = use_bias
         self.kernel_initializer = initializers.get(kernel_initializer)
 
+    @integerize_shape
     def build(self, input_shape):
         super(FeedForward, self).build(input_shape)
         output_dim = input_shape[-1]
-        if not isinstance(output_dim, int):
-            output_dim = output_dim.value
 
         self.dense_1 = Dense(units=self.units,
                              activation=self.activation,
@@ -549,10 +569,9 @@ class ConditionalRandomField(Layer):
         super(ConditionalRandomField, self).__init__(**kwargs)
         self.lr_multiplier = lr_multiplier  # 当前层学习率的放大倍数
 
+    @integerize_shape
     def build(self, input_shape):
         output_dim = input_shape[-1]
-        if not isinstance(output_dim, int):
-            output_dim = output_dim.value
         self.trans = self.add_weight(name='trans',
                                      shape=(output_dim, output_dim),
                                      initializer='glorot_uniform',
@@ -661,10 +680,9 @@ class MaximumEntropyMarkovModel(Layer):
         self.lr_multiplier = lr_multiplier  # 当前层学习率的放大倍数
         self.hidden_dim = hidden_dim  # 如果非None，则将转移矩阵低秩分解
 
+    @integerize_shape
     def build(self, input_shape):
         output_dim = input_shape[-1]
-        if not isinstance(output_dim, int):
-            output_dim = output_dim.value
 
         if self.hidden_dim is None:
             self.trans = self.add_weight(name='trans',
