@@ -21,6 +21,7 @@ class Transformer(object):
             hidden_act,  # FeedForward隐层的激活函数
             dropout_rate,  # Dropout比例
             embedding_size=None,  # 是否指定embedding_size
+            attention_key_size=None,  # Attention中Q,K的head_size
             keep_tokens=None,  # 要保留的词ID列表
             layers=None,  # 外部传入的Keras层
             name=None,  # 模型名称
@@ -33,6 +34,7 @@ class Transformer(object):
         self.num_hidden_layers = num_hidden_layers
         self.num_attention_heads = num_attention_heads
         self.attention_head_size = hidden_size // num_attention_heads
+        self.attention_key_size = attention_key_size or self.attention_head_size
         self.intermediate_size = intermediate_size
         self.dropout_rate = dropout_rate
         self.hidden_act = hidden_act
@@ -188,6 +190,30 @@ class Transformer(object):
             layer = self.layers[layer]
             weights = layer.trainable_weights
             values = [self.load_variable(checkpoint, v) for v in variables]
+
+            if isinstance(layer, MultiHeadAttention):
+                """如果key_size不等于head_size，则可以通过
+                正交矩阵将相应的权重投影到合适的shape。
+                """
+                count = 2
+                if layer.use_bias:
+                    count += 2
+                heads = self.num_attention_heads
+                head_size = self.attention_head_size
+                key_size = self.attention_key_size
+                W = np.linalg.qr(np.random.randn(key_size, head_size))[0].T
+                if layer.scaled_dot_product:
+                    W = W * key_size**0.25 / head_size**0.25
+                for i in range(count):
+                    w, v = weights[i], values[i]
+                    w_shape, v_shape = K.int_shape(w), v.shape
+                    if w_shape[-1] != v_shape[-1]:
+                        pre_shape = w_shape[:-1]
+                        v = v.reshape(pre_shape + (heads, head_size))
+                        v = np.dot(v, W)
+                        v = v.reshape(pre_shape + (heads * key_size, ))
+                        values[i] = v
+
             weight_value_pairs.extend(zip(weights, values))
 
         K.batch_set_value(weight_value_pairs)
@@ -303,6 +329,7 @@ class BERT(Transformer):
                       arguments=arguments,
                       heads=self.num_attention_heads,
                       head_size=self.attention_head_size,
+                      key_size=self.attention_key_size,
                       kernel_initializer=self.initializer,
                       name=attention_name)
         x = self.call(inputs=x,
@@ -526,6 +553,7 @@ class ALBERT(BERT):
                       arguments=arguments,
                       heads=self.num_attention_heads,
                       head_size=self.attention_head_size,
+                      key_size=self.attention_key_size,
                       kernel_initializer=self.initializer,
                       name=attention_name)
         x = self.call(inputs=x,
@@ -716,6 +744,7 @@ class NEZHA(BERT):
                       arguments=arguments,
                       heads=self.num_attention_heads,
                       head_size=self.attention_head_size,
+                      key_size=self.attention_key_size,
                       kernel_initializer=self.initializer,
                       name=attention_name)
         x = self.call(inputs=x,
@@ -863,6 +892,7 @@ class GPT2_ML(Transformer):
                       arguments=arguments,
                       heads=self.num_attention_heads,
                       head_size=self.attention_head_size,
+                      key_size=self.attention_key_size,
                       kernel_initializer=self.initializer,
                       name=attention_name)
         x = self.call(inputs=x,
@@ -1159,7 +1189,9 @@ class T5_Encoder(T5_Base):
                       arguments={'p_bias': 't5_relative'},
                       heads=self.num_attention_heads,
                       head_size=self.attention_head_size,
+                      key_size=self.attention_key_size,
                       use_bias=False,
+                      scaled_dot_product=False,
                       kernel_initializer=self.initializer,
                       name=attention_name)
         x = self.call(inputs=x,
@@ -1311,7 +1343,9 @@ class T5_Decoder(Transformer):
                       arguments={'a_mask': True, 'p_bias': 't5_relative'},
                       heads=self.num_attention_heads,
                       head_size=self.attention_head_size,
+                      key_size=self.attention_key_size,
                       use_bias=False,
+                      scaled_dot_product=False,
                       kernel_initializer=self.initializer,
                       name=self_attention_name)
         x = self.call(inputs=x,
@@ -1338,7 +1372,9 @@ class T5_Decoder(Transformer):
                       arguments={'a_mask': None, 'p_bias': 't5_relative'},
                       heads=self.num_attention_heads,
                       head_size=self.attention_head_size,
+                      key_size=self.attention_key_size,
                       use_bias=False,
+                      scaled_dot_product=False,
                       kernel_initializer=self.initializer,
                       name=cross_attention_name)
         x = self.call(inputs=x,
@@ -1567,13 +1603,14 @@ def build_transformer_model(config_path=None,
                             **kwargs):
     """根据配置文件构建模型，可选加载checkpoint权重
     """
-    config = kwargs
+    configs = {}
     if config_path is not None:
-        config.update(json.load(open(config_path)))
-    if 'max_position' not in config:
-        config['max_position'] = config.get('max_position_embeddings')
-    if 'dropout_rate' not in config:
-        config['dropout_rate'] = config.get('hidden_dropout_prob')
+        configs.update(json.load(open(config_path)))
+    configs.update(kwargs)
+    if 'max_position' not in configs:
+        configs['max_position'] = configs.get('max_position_embeddings')
+    if 'dropout_rate' not in configs:
+        configs['dropout_rate'] = configs.get('hidden_dropout_prob')
 
     model, application = model.lower(), application.lower()
 
@@ -1593,8 +1630,8 @@ def build_transformer_model(config_path=None,
         elif application == 'unilm':
             MODEL = extend_with_unified_language_model(MODEL)
 
-    transformer = MODEL(**kwargs)
-    transformer.build(**kwargs)
+    transformer = MODEL(**configs)
+    transformer.build(**configs)
 
     if checkpoint_path is not None:
         transformer.load_weights_from_checkpoint(checkpoint_path)
