@@ -232,7 +232,7 @@ def is_one_of(x, ys):
 class DataGenerator(object):
     """数据生成器模版
     """
-    def __init__(self, data, batch_size=32):
+    def __init__(self, data, batch_size=32, buffer_size=None):
         self.data = data
         self.batch_size = batch_size
         if hasattr(self.data, '__len__'):
@@ -241,6 +241,7 @@ class DataGenerator(object):
                 self.steps += 1
         else:
             self.steps = None
+        self.buffer_size = buffer_size or batch_size * 1000
 
     def __len__(self):
         return self.steps
@@ -248,13 +249,29 @@ class DataGenerator(object):
     def sample(self, random=False):
         """采样函数，每个样本同时返回一个is_end标记
         """
-        if random and self.steps is not None:
+        if random:
+            if self.steps is None:
 
-            def generator():
-                indices = list(range(len(self.data)))
-                np.random.shuffle(indices)
-                for i in indices:
-                    yield self.data[i]
+                def generator():
+                    caches, isfull = [], False
+                    for d in self.data:
+                        caches.append(d)
+                        if isfull:
+                            i = np.random.randint(len(caches))
+                            yield caches.pop(i)
+                        elif len(caches) == self.buffer_size:
+                            isfull = True
+                    while caches:
+                        i = np.random.randint(len(caches))
+                        yield caches.pop(i)
+
+            else:
+
+                def generator():
+                    indices = list(range(len(self.data)))
+                    np.random.shuffle(indices)
+                    for i in indices:
+                        yield self.data[i]
 
             data = generator()
         else:
